@@ -6,42 +6,93 @@ from tempfile import NamedTemporaryFile
 from unittest.case import TestCase
 
 import eventsourcing
-from eventsourcing.domain.model.events import (
-    assert_event_handlers_empty,
-    clear_event_handlers,
-)
-from eventsourcing.infrastructure.sqlalchemy.datastore import (
-    SQLAlchemyDatastore,
-    SQLAlchemySettings,
-)
-from eventsourcing.infrastructure.sqlalchemy.records import Base
+from eventsourcing.postgres import PostgresDatastore
+from eventsourcing.tests.ramdisk import tmpfile_uris
+from eventsourcing.tests.test_postgres import drop_postgres_table
 
 base_dir = dirname(dirname(os.path.abspath(eventsourcing.__file__)))
 
 
+class TestExample(TestCase):
+    def test(self):
+        from eventsourcing.domain import Aggregate, event
+
+        class World(Aggregate):
+            def __init__(self):
+                self.history = []
+
+            @event("SomethingHappened")
+            def make_it_so(self, what):
+                self.history.append(what)
+
+        from eventsourcing.application import Application
+
+        class Worlds(Application):
+            def create_world(self):
+                world = World()
+                self.save(world)
+                return world.id
+
+            def make_it_so(self, world_id, what):
+                world = self.repository.get(world_id)
+                world.make_it_so(what)
+                self.save(world)
+
+            def get_world_history(self, world_id):
+                world = self.repository.get(world_id)
+                return world.history
+
+        app = Worlds()
+        world_id = app.create_world()
+        app.make_it_so(world_id, "dinosaurs")
+        app.make_it_so(world_id, "trucks")
+        app.make_it_so(world_id, "internet")
+
+        history = app.get_world_history(world_id)
+        assert history == ["dinosaurs", "trucks", "internet"]
+
+
 class TestDocs(TestCase):
     def setUp(self) -> None:
-        assert_event_handlers_empty()
+        super().setUp()
+        self.uris = tmpfile_uris()
 
-    def tearDown(self):
-        clear_event_handlers()
-
-        # Need to drop the stored events, because the __main__#Order topics
-        # mess up the multiprocessing tests (because the Orders application
-        # has the same ID so events from one test cause another to fail).
-        os.environ[
-            "DB_URI"
-        ] = "mysql+pymysql://{}:{}@{}/eventsourcing?charset=utf8mb4&binary_prefix=true".format(
-            os.getenv("MYSQL_USER", "eventsourcing"),
-            os.getenv("MYSQL_PASSWORD", "eventsourcing"),
-            os.getenv("MYSQL_HOST", "127.0.0.1"),
+        db = PostgresDatastore(
+            "eventsourcing",
+            "127.0.0.1",
+            "5432",
+            "eventsourcing",
+            "eventsourcing",
         )
-        database = SQLAlchemyDatastore(settings=SQLAlchemySettings())
-        database.setup_connection()
-        # Might as well drop everything....
-        Base.metadata.drop_all(database._engine)
+        drop_postgres_table(db, "universe_events")
 
-        del os.environ["DB_URI"]
+    def tearDown(self) -> None:
+        self.clean_env()
+
+    def clean_env(self):
+        keys = [
+            "INFRASTRUCTURE_FACTORY",
+            "IS_SNAPSHOTTING_ENABLED",
+            "POSTGRES_DBNAME",
+            "POSTGRES_HOST",
+            "POSTGRES_PORT",
+            "POSTGRES_USER",
+            "POSTGRES_PASSWORD",
+            "POSTGRES_CONN_MAX_AGE",
+            "POSTGRES_PRE_PING",
+            "POSTGRES_LOCK_TIMEOUT",
+            "POSTGRES_IDLE_IN_TRANSACTION_SESSION_TIMEOUT",
+            "SQLITE_DBNAME",
+            "SQLITE_LOCK_TIMEOUT",
+            "CIPHER_TOPIC",
+            "CIPHER_KEY",
+            "COMPRESSOR_TOPIC",
+        ]
+        for key in keys:
+            try:
+                del os.environ[key]
+            except KeyError:
+                pass
 
     def test_readme(self):
         self._out = ""
@@ -51,10 +102,10 @@ class TestDocs(TestCase):
             self.skipTest("Skipped test, README file not found: {}".format(path))
         self.check_code_snippets_in_file(path)
 
-        path = join(base_dir, "README_example_with_axon.md")
-        if not os.path.exists(path):
-            self.skipTest("Skipped test, README file not found: {}".format(path))
-        self.check_code_snippets_in_file(path)
+        # path = join(base_dir, "README_example_with_axon.md")
+        # if not os.path.exists(path):
+        #     self.skipTest("Skipped test, README file not found: {}".format(path))
+        # self.check_code_snippets_in_file(path)
 
     def test_docs(self):
 
@@ -74,7 +125,13 @@ class TestDocs(TestCase):
                 if name in skipped:
                     continue
                 if name.endswith(".rst"):
-                    # if name.endswith('domainmodel.rst'):
+                    # if (
+                    #     name.endswith("persistence.rst")
+                    #     or name.endswith("domain.rst")
+                    #       or name.endswith("application.rst")
+                    #     or name.endswith("system.rst")
+                    #     or name.endswith("examples.rst")
+                    # ):
                     # if name.endswith('quick_start.rst'):
                     # if name.endswith('aggregates_in_ddd.rst'):
                     # if name.endswith('example_application.rst'):
@@ -100,7 +157,6 @@ class TestDocs(TestCase):
             # print("Testing code snippets in file: {}".format(path))
             try:
                 self.check_code_snippets_in_file(path)
-                assert_event_handlers_empty()
             except self.failureException as e:
                 failures.append(e)
                 failed.append(path)
@@ -112,7 +168,7 @@ class TestDocs(TestCase):
                 print("PASS")
                 print("")
             finally:
-                os.environ["SALT_FOR_DATA_INTEGRITY"] = ""
+                self.clean_env()
 
         print("{} failed, {} passed".format(len(failed), len(passed)))
 
@@ -129,14 +185,21 @@ class TestDocs(TestCase):
         is_md = False
         is_rst = False
         last_line = ""
+        is_literalinclude = False
+        module = ""
         with open(doc_path) as doc_file:
-            for orig_line in doc_file:
+            for line_index, orig_line in enumerate(doc_file):
+                # print("Line index:", line_index)
+                # print("Orig line:", orig_line)
+                # print("Last line:", last_line)
+
                 line = orig_line.strip("\n")
                 if line.startswith("```python"):
                     # Start markdown code block.
                     if is_rst:
                         self.fail(
-                            "Markdown code block found after restructured text block in same file."
+                            "Markdown code block found after restructured text block "
+                            "in same file."
                         )
                     is_code = True
                     is_md = True
@@ -145,7 +208,7 @@ class TestDocs(TestCase):
                 elif is_code and is_md and line.startswith("```"):
                     # Finish markdown code block.
                     if not num_code_lines_in_block:
-                        self.fail("No lines of code in block")
+                        self.fail(f"No lines of code in block: {line_index + 1}")
                     is_code = False
                     line = ""
                 elif is_code and is_rst and line.startswith("```"):
@@ -153,25 +216,47 @@ class TestDocs(TestCase):
                     self.fail(
                         "Restructured text block terminated with markdown format '```'"
                     )
-                elif (
-                    line.startswith(".. code:: python")
-                    and "exclude-when-testing" not in last_line
+                elif line.startswith(".. code-block:: python") or (
+                    line.strip() == ".." and "include-when-testing" in last_line
                 ):
                     # Start restructured text code block.
                     if is_md:
                         self.fail(
-                            "Restructured text code block found after markdown block in same file."
+                            "Restructured text code block found after markdown block "
+                            "in same file."
                         )
                     is_code = True
                     is_rst = True
                     line = ""
                     num_code_lines_in_block = 0
+                elif line.startswith(".. literalinclude::"):
+                    is_literalinclude = True
+                    module = line.strip().split(" ")[-1]  # get the file path
+                    module = module[:-3]  # remove the '.py' from the end
+                    module = module.lstrip("./")  # remove all the ../../..
+                    module = module.replace("/", ".")  # swap dots for slashes
+                    line = ""
+
+                elif is_literalinclude:
+                    if "pyobject" in line:
+                        # Assume ".. literalinclude:: ../../xxx/xx.py"
+                        # Or ".. literalinclude:: ../xxx/xx.py"
+                        # Assume "    :pyobject: xxxxxx"
+                        pyobject = line.strip().split(" ")[-1]
+                        statement = f"from {module} import {pyobject}"
+                        line = statement
+                    elif not line.strip():
+                        is_literalinclude = False
+                        module = ""
+
                 elif is_code and is_rst and line and not line.startswith(" "):
                     # Finish restructured text code block.
                     if not num_code_lines_in_block:
-                        self.fail("No lines of code in block")
+                        self.fail(f"No lines of code in block: {line_index + 1}")
                     is_code = False
                     line = ""
+                elif ":emphasize-lines:" in line:
+                    line = ""
                 elif is_code:
                     # Process line in code block.
                     if is_rst:
@@ -192,6 +277,7 @@ class TestDocs(TestCase):
                 else:
                     line = ""
                 lines.append(line)
+                # if orig_line.strip():
                 last_line = orig_line
 
         print("{} lines of code in {}".format(num_code_lines, doc_path))
